# Generated by Django 3.0.3 on 2020-02-21 04:51

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ScopedPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('scope', models.TextField()),
                ('exclude', models.BooleanField(default=False, help_text="Whether this should be an exclusive permission, meaning that if scope is 'user:update' and exclude is True then users with this usertype will not be able to update users, even their own.")),
                ('exact', models.BooleanField(default=False, help_text='If checked, the permission needs an exact match to count. In other words, it does not work recursively as standard scoped permissions.')),
            ],
            options={
                'unique_together': {('scope', 'exclude', 'exact')},
            },
        ),
    ]
